// Generated by CoffeeScript 1.10.0
(function() {
  var Logger, Path, Track, clone, deepMap, domain, fixPathPiece, fs, id3, mkdirp, objTypeof, ref, request, sformat;

  fs = require("fs");

  mkdirp = require("mkdirp");

  id3 = require("node-id3");

  domain = require("domain");

  request = require("request");

  Path = require("path");

  Logger = require("./log");

  Logger = new Logger();

  clone = require("clone");

  sformat = require("string-format");

  ref = require("./util"), objTypeof = ref.objTypeof, deepMap = ref.deepMap, fixPathPiece = ref.fixPathPiece;

  Track = (function() {
    function Track(uri, config, callback) {
      this.uri = uri;
      this.config = config;
      this.callback = callback;
      this.track = {};
      this.file = {};
      this.retryCounter = 0;
    }

    Track.setSpotify = function(spotify) {
      this.spotify = spotify;
    };

    Track.prototype.process = function() {
      return this.constructor.spotify.get(this.uri, (function(_this) {
        return function(err, track) {
          var error;
          if (err) {
            return typeof _this.callback === "function" ? _this.callback(err) : void 0;
          }
          _this.track = track;
          try {
            return _this.createDirs();
          } catch (error) {
            err = error;
            Logger.Error("Error on track: \"" + _this.track.artist[0].name + " - " + _this.track.name + "\" : " + err + " \n\n" + err.stack);
            return typeof _this.callback === "function" ? _this.callback() : void 0;
          }
        };
      })(this));
    };

    Track.prototype.createDirs = function() {
      var _path, err, error, fields, fixStrg, pathFormat, stats, trackCopy;
      this.config.directory = Path.resolve(this.config.directory);
      pathFormat = this.config.folder && typeof this.config.folder === 'string' ? this.config.folder : "{artist.name} - {track.name}";
      trackCopy = clone(this.track);
      trackCopy.name = trackCopy.name.replace(/\//g, " - ");
      fixStrg = (function(_this) {
        return function(obj) {
          if (objTypeof(obj) === "[object String]") {
            obj = obj.replace(/\//g, "-");
            if (_this.config.onWindows) {
              obj = fixPathPiece(obj);
            }
          }
          return obj;
        };
      })(this);
      deepMap.call({
        fn: fixStrg
      }, trackCopy);
      fields = {
        track: trackCopy,
        artist: trackCopy.artist[0],
        album: trackCopy.album
      };
      fields.album.year = fields.album.date.year;
      try {
        _path = sformat(pathFormat, fields);
      } catch (error) {
        err = error;
        Logger.Error("Invalid path format: " + err, 1);
        return typeof this.callback === "function" ? this.callback() : void 0;
      }
      if (!_path.endsWith(".mp3")) {
        _path += ".mp3";
      }
      this.file.path = Path.join(this.config.directory, _path);
      this.file.directory = Path.dirname(this.file.path);
      if (fs.existsSync(this.file.path)) {
        stats = fs.statSync(this.file.path);
        if (stats.size !== 0) {
          Logger.Info("Already downloaded: " + this.track.artist[0].name + " - " + this.track.name, 1);
          return typeof this.callback === "function" ? this.callback() : void 0;
        }
      }
      if (!fs.existsSync(this.file.directory)) {
        mkdirp.sync(this.file.directory);
      }
      Logger.Log("Downloading: " + this.track.artist[0].name + " - " + this.track.name, 1);
      this.downloadCover();
      return this.downloadFile();
    };

    Track.prototype.downloadCover = function() {
      var coverPath, coverUrl, image, images, ref1, ref2;
      coverPath = this.file.path + ".jpg";
      images = (ref1 = this.track.album.coverGroup) != null ? ref1.image : void 0;
      image = (ref2 = images != null ? images[2] : void 0) != null ? ref2 : images != null ? images[0] : void 0;
      if (!image) {
        Logger.Error("Can't download cover: " + this.track.artist[0].name + " - " + this.track.name, 2);
        return;
      }
      coverUrl = "" + image.uri;
      request.get(coverUrl).on("error", (function(_this) {
        return function(err) {
          return Logger.Error("Error while downloading cover: " + err);
        };
      })(this)).pipe(fs.createWriteStream(coverPath));
      return Logger.Success("Cover downloaded: " + this.track.artist[0].name + " - " + this.track.name, 2);
    };

    Track.prototype.downloadFile = function() {
      var d;
      d = domain.create();
      d.on("error", (function(_this) {
        return function(err) {
          Logger.Error("Error received: " + err, 2);
          if (("" + err).indexOf("Rate limited") > -1) {
            Logger.Info(err + " ... { Retrying in 10 seconds }", 2);
            if (_this.retryCounter < 2) {
              _this.retryCounter++;
              return setTimeout(_this.downloadFile, 10000);
            } else {
              Logger.Error("Unable to download song. Continuing", 2);
              return typeof _this.callback === "function" ? _this.callback() : void 0;
            }
          } else {
            return typeof _this.callback === "function" ? _this.callback() : void 0;
          }
        };
      })(this));
      return d.run((function(_this) {
        return function() {
          var err, error, out;
          out = fs.createWriteStream(_this.file.path);
          try {
            return _this.track.play().pipe(out).on("finish", function() {
              Logger.Success("Done: " + _this.track.artist[0].name + " - " + _this.track.name, 2);
              return _this.writeMetadata();
            });
          } catch (error) {
            err = error;
            Logger.Error("Error while downloading track! " + err, 2);
            return typeof _this.callback === "function" ? _this.callback() : void 0;
          }
        };
      })(this));
    };

    Track.prototype.writeMetadata = function() {
      var meta;
      meta = {
        artist: this.track.artist[0].name,
        album: this.track.album.name,
        title: this.track.name,
        year: "" + this.track.album.date.year,
        trackNumber: "" + this.track.number,
        image: this.file.path + ".jpg"
      };
      id3.write(meta, this.file.path);
      fs.unlink(meta.image);
      return typeof this.callback === "function" ? this.callback() : void 0;
    };

    return Track;

  })();

  module.exports = Track;

}).call(this);
